﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace FoundationLib.Data
{
	/// <summary>
	/// Provides session functionality.
	/// </summary>
	public interface ISession : ITransactionProvider, IDisposable
	{
		/// <summary>
		/// Closes the current session.
		/// </summary>
		void Close();

		/// <summary>
		/// Flushes the contents of the current session by synchronising changes with the data store.
		/// </summary>
		void Flush();
		
		/// <summary>
		/// Inserts the given instance into the data store.
		/// </summary>
		/// <param name="instance">The instance to store</param>
		void Save(object instance);

		/// <summary>
		/// Deletes the given instance from the data store.
		/// </summary>
		/// <param name="instance">The instance to delete</param>
		void Delete(object instance);

		/// <summary>
		/// Removes the given instance from the repository associated with the current session.
		/// </summary>
		/// <param name="instance">The instance to remove</param>
		void Evict(object instance);

		/// <summary>
		/// Updates the state of the given instance in the data store.
		/// </summary>
		/// <param name="instance">The instance to update</param>
		void Update(object instance);

		/// <summary>
		/// Checks if this instance exists in the data store and performs an insert or 
		/// update operation.
		/// </summary>
		/// <param name="instance">The instance to save or update</param>
		void SaveOrUpdate(object instance);

		/// <summary>
		/// Checks whether the given instance is contained in the current session.
		/// </summary>
		/// <param name="instance">The instance</param>
		/// <returns>True if the instance is contained, false else</returns>
		bool Contains(object instance);

		/// <summary>
		/// Provides queryable access to an object of the given type <typeparamref name="T"/>.
		/// </summary>
		/// <typeparam name="T">The type of entity</typeparam>
		/// <returns>An <see cref="IQueryable{T}"/> instance for query operations</returns>
		IQueryable<T> Query<T>();
	}
}
